from Crypto.Cipher import DES
from Crypto.Util.Padding import pad, unpad
import itertools
import time

# Sample plaintext to encrypt
plaintext = b"This is a secret message."

# The key we will use for this demonstration (Unknown to the attacker)
true_key = b"KEY12345"  # 8 bytes for DES

# DES encryption using the secret key
cipher = DES.new(true_key, DES.MODE_ECB)
ciphertext = cipher.encrypt(pad(plaintext, DES.block_size))
print(f"\nCiphertext: {ciphertext.hex()}\n")

# Brute-force attack to find the key
start_time = time.time()
found_key = None

# Iterate over all possible 56-bit keys (7 bytes + 1 parity bit for each)
for key_candidate in itertools.product(range(256), repeat=8):
    key_candidate = bytes(key_candidate)

    # Try decrypting the ciphertext with the candidate key
    try:
        cipher = DES.new(key_candidate, DES.MODE_ECB)
        decrypted_message = unpad(cipher.decrypt(ciphertext), DES.block_size)

        if decrypted_message == plaintext:
            found_key = key_candidate
            break
    except:
        continue

# Display the results
if found_key:
    elapsed_time = time.time() - start_time
    print(f"Key Found: {found_key.decode(errors='ignore')}")
    print(f"Key (Hex): {found_key.hex()}")
    print(f"Decrypted Message: {decrypted_message.decode()}")
    print(f"Time Taken: {elapsed_time:.2f} seconds")
else:
    print("Key not found.")