import bcrypt
from functools import reduce
import itertools
from time import time


def bruteforce(target_hash, pwd_length):

    seed = "aeosrnidlctumpbgqvyhfzjxwk"                        # lowercase
    # seed = "aeosrnidlctumpbgqvyhfzjxwk" + "1234567890"       # uppercase + numbers
    seed_bytes = list(map(ord, seed))
    print("seed_bytes = %s" % seed_bytes)

    # Possible are: permutations, combinations or product
    attempts = 0
    start = time()
    for word_bytes in itertools.product(seed_bytes, repeat=pwd_length):

        word_string = reduce(lambda x, y: x+y, map(chr, word_bytes))            # word_bytes to string

        # This does the check, extracting cost factor and salt from target_hsh and computing hash of candidate pwd
        hash_ = bcrypt.checkpw(word_string.encode(), target_hash)

        if hash_:
            print("\n==> PASSWORD CRACKED: word = %s | hash = %s" % (word_string, hash_))
            break

        # Update on progress: must increase the frequency a lot with respect to MD5!
        if attempts % (10) == 0:
            elapsed = time() - start
            rate = attempts / elapsed
            print("word = %s | hash = %s | attempts = %d | elapsed = %d | rate = %.2f" % (word_string, hash_, attempts, elapsed, rate))

        attempts += 1


if __name__ == "__main__":

    SECRET_PASSWORD = "letmein"       # The attacker does not know this password and wants to guess it
    HASH_TO_BREAK = bcrypt.hashpw(SECRET_PASSWORD.encode(), bcrypt.gensalt(rounds=12))

    # Let's assume that the attacker knows the length of the password. Without this information, the attack would have
    # to loop all password lengths
    TARGET_LENGTH = len(SECRET_PASSWORD)

    # The cost factor is not a secret: one can retrieve it from the hash itself
    # COST_FACTOR = int(HASH_TO_BREAK.decode('utf-8')[4:6])

    bruteforce(target_hash=HASH_TO_BREAK, pwd_length=TARGET_LENGTH)