# Simple MD5 Bruteforce work with dynamic dictionary on memory, the length of the password is known
from functools import reduce
import hashlib
import itertools
from time import time


def bruteforce(target_hash, pwd_length):

    # Seeding an alphabet in a frequency-based order (like "aeosrnidlctumpbgqvyhfzjxwk") rather than the standard
    # "abcdefghijklmnopqrstuvwxyz" can improve the performance of a brute-force attack because it prioritizes trying
    # more likely combinations first. This approach leverages probabilistic optimization to reduce the average time to find a match.

    seed = "aeosrnidlctumpbgqvyhfzjxwk"                        # lowercase
    # seed = "aeosrnidlctumpbgqvyhfzjxwk" + "1234567890"       # uppercase + numbers
    seed_bytes = list(map(ord, seed))
    print("seed_bytes = %s" % seed_bytes)

    # Possible are: permutations, combinations or product
    attempts = 0
    start = time()
    for word_bytes in itertools.product(seed_bytes, repeat=pwd_length):

        word_string = reduce(lambda x, y: x+y, map(chr, word_bytes))            # word_bytes to string
        hash_ = hashlib.md5(word_string.encode('utf-8')).hexdigest()            # MD5 of word_bytes

        if hash_ == target_hash:
            print("\n==> PASSWORD CRACKED: word = %s | hash = %s" % (word_string, hash_))
            break

        # Update on progress
        if attempts % (1 << 20) == 0:
            elapsed = time() - start
            rate = attempts / elapsed
            print("word = %s | hash = %s | attempts = %d | elapsed = %d | rate = %.2f" % (word_string, hash_, attempts, elapsed, rate))

        attempts += 1


if __name__ == "__main__":

    SECRET_PASSWORD = "letmein"       # The attacker does not know this password and wants to guess it
    HASH_TO_BREAK = hashlib.md5(SECRET_PASSWORD.encode('utf-8')).hexdigest()    # The attacked has the hashed password
    print(HASH_TO_BREAK)
    # Let's assume that the attacker knows the length of the password. Without this information, the attack would have
    # to loop all password lengths
    TARGET_LENGTH = len(SECRET_PASSWORD)

    bruteforce(target_hash=HASH_TO_BREAK, pwd_length=TARGET_LENGTH)