
# CWE-326: Inadequate Encryption Strength
# MD5 is flawed and susceptible to collision attacks. It is possible to generate two different inputs that
# produce the same hash output.

import hashlib

# Known colliding inputs (hex-encoded for demonstration)
input1 = "TEXTCOLLBYfGiJUETHQ4hAcKSMd5zYpgqf1YRDhkmxHkhPWptrkoyz28wnI9V0aHeAuaKnak"
input2 = "TEXTCOLLBYfGiJUETHQ4hEcKSMd5zYpgqf1YRDhkmxHkhPWptrkoyz28wnI9V0aHeAuaKnak"

# Compare the two input strings
for i in range(len(input1)):
    print(i+1, input1[i], input2[i], '*' if input1[i] != input2[i] else ' ')

# Or do it faster: https://cyberchef.io
# or similar: https://tool.cyberchef.dev. BUT: the former runs entirely on browser (or locally) while the latter,
# more appealing, runs remotely on a server that gets your input data and configurations, not prioritizing privacy
# and security
# Note: diff tool is useful for several tasks in CyberSecurity: comparing hashes, cookies, requests and responses from
# an HTTP server, fingerprints, original vs manipulated file (violated integrity in CIA triad) ...

# Calculate the MD5 hashes of both inputs
hash1 = hashlib.md5(input1.encode('utf-8')).hexdigest()
hash2 = hashlib.md5(input2.encode('utf-8')).hexdigest()

# Print results
print(f"Input 1 MD5: {hash1}")
print(f"Input 2 MD5: {hash2}")
print(f"Collision Detected: {hash1 == hash2}")