import requests


# Base URL of the insecure web application
base_url = "http://127.0.0.1:5001/"

TARGET_FORMAT = "xml"

# Dictionary of common file names to try
common_files = [
    "config.xml",
    "backup.xml",
    "settings.xml",
    "database.xml",
    "credentials.xml",
    "admin.xml",
    "web.config",
    "config_old.xml",
    "config.bak",
    "data.xml",
]

# Or load a dictionary of common file names from a file
with open("resources/commonfiles.txt", "r") as f:
    common_files = f.readlines()
    common_files = [x.strip().replace("\n", "") for x in common_files]

common_files = [f'{x}.{TARGET_FORMAT}' for x in common_files]
print(F"Loaded {len(common_files)} common file names from file.")

# Store results of found files
found_files = {}

# Loop through the dictionary of files
for file_name in common_files:
    # Construct the full URL
    url = f"{base_url}{file_name}"
    try:
        # Make the HTTP GET request
        response = requests.get(url)

        # If the status code is 200, the file exists
        if response.status_code == 200:
            print(f"✅ Found: {url}")
            found_files[file_name] = response.text  # Store the file's content
        else:
            print(f"❌ Not Found: {url} (Status Code: {response.status_code})")

    except requests.RequestException as e:
        print(f"Error making request to {url}: {e}")

# Display found files
if found_files:
    print("\nFiles Found:")
    for file_name, content in found_files.items():
        print(f"\nFile: {file_name}")
        print(f"Content Snippet:\n{content[:200]}")  # Show first 200 characters of the file content
else:
    print("\nNo files were found from the dictionary attack.")